@@ -13,8 +13,8 @@
 #include <boost/uuid/uuid_io.hpp>         // streaming operators etc.
 #include <boost/algorithm/string/replace.hpp>
 
-#include "nlohmann/json.hpp"
 #include <internal/uuid.h>
+#include <internal/string_tools.h>
 
 #include "Config.hh"
 //#ifndef ENABLE_JUPYTER
@@ -94,9 +94,47 @@ PYBIND11_EMBEDDED_MODULE(cadabra2_internal, m)
 	pybind11::class_<Server>(m, "Server")
 	.def("send", &Server::send)
 	.def("handles", &Server::handles)
-	.def("architecture", &Server::architecture);
+	.def("architecture", &Server::architecture)
+	.def("send_progress_update", &Server::send_progress_update);
 	}
 
+std::string parse_error(const std::string& error, const std::string& input)
+{
+	try {
+		// Find syntax errors
+		std::regex syntax_error(R"(SyntaxError: \('([^']+)', \('<string>', (\d+), (\d+),.*)");
+		std::smatch sm;
+		if (std::regex_match(error, sm, syntax_error)) {
+			std::string error_type = sm[1];
+			size_t line_no = stoi(sm[2]) - 1;
+			size_t col_no = stoi(sm[3]);
+			return
+				"SyntaxError: " + error_type + "\n" +
+				"Line " + std::to_string(line_no) + ", column " + std::to_string(col_no) + "\n" +
+				nth_line(input, line_no - 1) + "\n" +
+				std::string(col_no > 1 ? col_no - 2 : 0, ' ') + "^";
+		}
+
+		// Find other errors
+		std::regex exception_name(R"(([a-zA-Z_][a-zA-Z0-9_]*):.*)");
+		std::string first_line = nth_line(error, 0);
+		if (std::regex_match(first_line, sm, exception_name)) {
+			std::string name = sm[1];
+			std::regex line_no(R"(<string>\((\d+)\): <module>)");
+			std::smatch lm;
+			if (std::regex_search(error, lm, line_no)) {
+				size_t l = stoi(lm[1]) - 1;
+				return std::regex_replace(error, line_no, "Notebook Cell (Line " + std::to_string(l) + "): " + nth_line(input, l - 1));
+			}
+		}
+
+		return error;
+	}
+	catch (std::exception& e) {
+		return error;
+	}
+}
+
 void Server::init()
 	{
 	started=false;
@@ -106,7 +144,7 @@ void Server::init()
 
 	// Make the C++ CatchOutput class visible on the Python side.
 
-	auto python_path = std::string(PYTHON_SITE_PATH);
+	auto python_path = std::string(cadabra::install_prefix() + "/lib/python" + std::to_string(PY_MAJOR_VERSION) + "." + std::to_string(PY_MINOR_VERSION) + "/" + std::string(PYTHON_SITE_DIST));
 
 	std::string stdOutErr =
 	   "import sys\n"
@@ -197,7 +235,7 @@ std::string Server::run_string(const std::string& blk, bool handle_output)
 		//    https://github.com/pybind/pybind11/issues/1490
 		// Note: the restore() has the side effect of making the
 		// error come back on any future pybind11::exec() call.
-		std::string reason=ex.what();
+		std::string reason=parse_error(ex.what(), newblk);
 		ex.restore();
 		throw std::runtime_error(reason);
 		}
@@ -331,9 +369,13 @@ void Server::stop_block()
 	PyGILState_Release(state);
 	}
 
-Server::Block::Block(websocketpp::connection_hdl h, const std::string& str, uint64_t id_)
-	: hdl(h), input(str), cell_id(id_)
+Server::Block::Block(websocketpp::connection_hdl h, const std::string& str, uint64_t id_, const std::string& msg_type_)
+	: hdl(h), msg_type(msg_type_), input(str), cell_id(id_)
 	{
+	nlohmann::json content, header;
+	response["header"]=header;
+	response["content"]=content;
+	response["msg_type"]=msg_type;
 	}
 
 void Server::on_message(websocketpp::connection_hdl hdl, WebsocketServer::message_ptr msg)
@@ -385,7 +427,10 @@ void Server::dispatch_message(websocketpp::connection_hdl hdl, const std::string
 		// std::cerr << code << std::endl;
 		uint64_t id = header.value("cell_id", uint64_t(0));
 		std::unique_lock<std::mutex> lock(block_available_mutex);
-		block_queue.push(Block(hdl, code, id));
+		Block block(hdl, code, id, msg_type);
+		block.response["header"]["parent_origin"]="client";
+		block.response["header"]["parent_id"]=id;
+		block_queue.push(block);
 		block_available.notify_one();
 		}
 	else if(msg_type=="execute_interrupt") {
@@ -402,31 +447,57 @@ void Server::dispatch_message(websocketpp::connection_hdl hdl, const std::string
 		stop_block();
 		std::queue<Block> empty;
 		std::swap(block_queue, empty);
-
 		}
 	else if(msg_type=="complete") {
+		// Schedule a block which runs code to complete the given string.
 		std::string str=root["string"].get<std::string>();
 		int alternative=root["alternative"].get<int>();
 		std::string todo="print(__cdbkernel__.completer.complete(\""+str+"\", "+std::to_string(alternative)+"))";
-		// std::cerr << todo << std::endl;
-		std::string res=run_string(todo, true);
-		// FIXME: need a better way to get the result out of python, so we can spot None
-		// while keeping the possibility to complete 'No' -> 'None'.
-		if(res.size()>0 && res[res.size()-1]=='\n')
-			res=res.substr(0, res.size()-1);
-		if(res=="None")
-			res="";
-		//std::cerr << res << "|" << std::endl;
-		on_complete_finished(hdl, header["cell_id"].get<uint64_t>(), root["position"].get<int>(), alternative, str, res);
+
+		uint64_t id = header.value("cell_id", uint64_t(0));
+		Block blk(hdl, todo, id, "completed");
+		blk.response["header"]["cell_id"]=id;
+		blk.response["content"]["original"]=str;
+		blk.response["content"]["position"]=root["position"].get<int>();
+		blk.response["content"]["alternative"]=alternative;
+
+		std::unique_lock<std::mutex> lock(block_available_mutex);
+		block_queue.push(blk);
+		block_available.notify_one();
 		}
 	else if(msg_type=="exit") {
 		exit(-1);
 		}
 	}
 
-void Server::on_block_finished(Block blk)
+void Server::on_block_finished(Block block)
 	{
-	send(blk.output, "output", 0, true); // last in sequence
+	auto& header  = block.response["header"];
+	auto& content = block.response["content"];
+
+	if(block.msg_type=="completed") {
+		// FIXME: need a better way to get the result out of python, so we can spot None
+		// while keeping the possibility to complete 'No' -> 'None'.
+		std::string res=block.output;
+		if(res.size()>0 && res[res.size()-1]=='\n')
+			res=res.substr(0, res.size()-1);
+		if(res=="None")
+			res="";
+		block.response["content"]["completed"]=res;
+		}
+	else {
+		header["cell_origin"]="server";
+		header["cell_id"]=cadabra::generate_uuid<uint64_t>();
+		header["time_total_microseconds"]=std::to_string(server_stopwatch.seconds()*1e6L + server_stopwatch.useconds());
+		header["time_sympy_microseconds"]=std::to_string(sympy_stopwatch.seconds()*1e6L  + sympy_stopwatch.useconds());
+		header["last_in_sequence"]=true;
+		content["output"]=block.output;
+		block.response["msg_type"]="output";
+		}
+
+	std::ostringstream str;
+	str << block.response << std::endl;
+	send_json(str.str());
 	}
 
 bool Server::handles(const std::string& otype) const
@@ -437,10 +508,11 @@ bool Server::handles(const std::string& otype) const
 
 uint64_t Server::send(const std::string& output, const std::string& msg_type, uint64_t parent_id, bool last)
 	{
-	//	if(msg_type=="output")
-	//		std::cerr << "Cell " << msg_type << " timing: " << server_stopwatch << " (in python: " << sympy_stopwatch << ")" << std::endl;
-	// Make a JSON message.
-	nlohmann::json json, content, header;
+	// This is the function exposed to the Python side; not used
+	// directly in the server to send block output back to the client
+	// (that's all handled by on_block_finished above).
+
+	nlohmann::json json, header, content;
 
 	auto return_cell_id=cadabra::generate_uuid<uint64_t>();
 	if(parent_id==0)
@@ -454,6 +526,7 @@ uint64_t Server::send(const std::string& output, const std::string& msg_type, ui
 	header["time_total_microseconds"]=std::to_string(server_stopwatch.seconds()*1e6L + server_stopwatch.useconds());
 	header["time_sympy_microseconds"]=std::to_string(sympy_stopwatch.seconds()*1e6L  + sympy_stopwatch.useconds());
 	header["last_in_sequence"]=last;
+
 	content["output"]=output;
 
 	json["header"]=header;
@@ -468,6 +541,29 @@ uint64_t Server::send(const std::string& output, const std::string& msg_type, ui
 	return return_cell_id;
 	}
 
+void Server::send_progress_update(const std::string& msg, int n, int total)
+	{
+	nlohmann::json json, content, header;
+
+	header["parent_id"] = 0;
+	header["parent_origin"] = "client";
+	header["cell_id"] = 0;
+	header["cell_origin"] = "server";
+
+	content["msg"] = msg;
+	content["n"] = n;
+	content["total"] = total;
+
+	json["header"] = header;
+	json["content"] = content;
+	json["msg_type"] = "progress_update";
+
+	std::ostringstream str;
+	str << json << std::endl;
+
+	send_json(str.str());
+	}
+
 void Server::send_json(const std::string& msg)
 	{
 	//	std::cerr << "*** sending message " << msg << std::endl;
@@ -501,29 +597,6 @@ void Server::on_block_error(Block blk)
 	wserver.send(blk.hdl, str.str(), websocketpp::frame::opcode::text);
 	}
 
-void Server::on_complete_finished(websocketpp::connection_hdl hdl, uint64_t id, int pos, int alternative, std::string original, std::string completed)
-	{
-//	std::lock_guard<std::mutex> lock(ws_mutex); // Called from the receiving thread!
-
-	// Make a JSON message.
-	nlohmann::json json, content, header;
-
-	header["cell_id"]=id;
-	json["msg_type"]="completed";
-	content["original"]=original;
-	content["completed"]=completed;
-	content["position"]=pos;
-	content["alternative"]=alternative;
-	json["header"]=header;
-	json["content"]=content;
-
-	std::ostringstream str;
-	str << json << std::endl;
-	// std::cerr << "cadabra-server: sending completion, " << str.str() << std::endl;
-
-	wserver.send(hdl, str.str(), websocketpp::frame::opcode::text);
-	}
-
 void Server::on_kernel_fault(Block blk)
 	{
 	std::lock_guard<std::mutex> lock(ws_mutex);
@@ -588,4 +661,3 @@ void Server::run(int port, bool eod)
 		std::cerr << "cadabra-server: websocket exception " << ex.code() << " " << ex.what() << std::endl;
 		}
 	}
-
