@@ -12,13 +12,15 @@
 # fi
 
 cd ${HOME}/cadabra2
-sudo ls 
+sudo ls
+rm -f config/install_script.iss config/pre_install.rtf
 git pull origin
 git checkout ${CDB_TAG}
 sudo rm -Rf build/*
 mkdir build
 cd build
 if [ -f /etc/redhat-release ]; then
+	 sudo dnf -y install rpm-build
     centos="`cat /etc/redhat-release | grep CentOS`"
     scilin="`cat /etc/redhat-release | grep Scientific`"
     if [ -n "${centos}" -o -n "${scilin}" ]; then
