@@ -25,7 +25,6 @@ You should have received a copy of the GNU General Public License
 #include <sstream>
 #include <locale>
 #include <codecvt>
-#include <pcrecpp.h>
 
 #include <boost/functional/hash.hpp>
 
@@ -41,6 +40,11 @@ namespace cadabra {
 		return mul.get_num().get_si();
 		}
 
+	double to_double(multiplier_t mul)
+		{
+		return mul.get_d();
+		}
+
 	std::string to_string(long num)
 		{
 		std::ostringstream str;
