@@ -2,6 +2,7 @@
 #include "Kernel.hh"
 #include "properties/Indices.hh"
 #include "properties/Integer.hh"
+#include "properties/Coordinate.hh"
 #include "Exceptions.hh"
 #include "Functional.hh"
 
@@ -74,6 +75,9 @@ bool Indices::parse(Kernel& kernel, std::shared_ptr<Ex> ex, keyval_t& keyvals)
 					is_number=false;
 					break;
 					}
+			// FIXME: the above assumes no gap in the integer range, which is
+			// not always guaranteed, and will then break the Integer property
+			// assignment as the latter can only deal with continuous ranges.
 			if(is_number) {
 				Ex from(values[0]), to(values[values.size()-1]);
 //				std::cerr << "Injecting Integer property" << std::endl;
@@ -108,8 +112,37 @@ void Indices::latex(std::ostream& str) const
 		}
 	}
 
+void Indices::validate(const Kernel& k, const Ex& ex) const
+	{
+	do_list(ex, ex.begin(), [&k](Ex::iterator i) {
+									if(k.properties.get<Coordinate>(i))
+										throw ConsistencyException("Object already has a Coordinate property attached to it.");
+									return true;
+									}
+		);
+	}
+
+
 void Indices::collect_index_values(Ex::iterator ind_values)
 	{
+	if(*ind_values->name=="\\sequence") {
+		// We have been given a sequence, not a list.
+		// FIXME: guard against errors or large ranges.
+		auto ch = Ex::begin(ind_values);
+		size_t start = to_long(*ch->multiplier);
+		++ch;
+		size_t end   = to_long(*ch->multiplier);
+		if(end<start)
+			throw ArgumentException("Index range must be increasing.");
+		if(end-start > 100)
+			throw ArgumentException("Number of index values larger than 100, probably a typo.");
+			
+		for(size_t i=start; i<=end; ++i) {
+			values.push_back(Ex(i));
+			}
+		return;
+		}
+	
 	Ex tmp;
 	cadabra::do_list(tmp, ind_values, [&](Ex::iterator ind) {
 		values.push_back(Ex(ind));
