@@ -95,6 +95,42 @@ namespace cadabra {
 //			}
 		}
 
+   Ex_ptr Ex_join(const Ex_ptr ex1, const Ex_ptr ex2)
+		{
+		if (ex1->size() == 0) return ex2;
+		if (ex2->size() == 0) return ex1;
+
+		bool comma1 = (*ex1->begin()->name == "\\comma");
+		bool comma2 = (*ex2->begin()->name == "\\comma");
+
+		if(comma1 || comma2) {
+			if (comma1) {
+				auto ret = std::make_shared<Ex>(*ex1);
+				auto loc = ret->append_child(ret->begin(), ex2->begin());
+				if (comma2)
+					ret->flatten_and_erase(loc);
+				return ret;
+				}
+			else {
+				auto ret = std::make_shared<Ex>(ex2->begin());
+				auto loc = ret->prepend_child(ret->begin(), ex1->begin());
+				if (comma1)
+					ret->flatten_and_erase(loc);
+				return ret;
+				}
+			}
+		else {
+			auto ret = std::make_shared<Ex>(*ex1);
+			if (*ret->begin()->name != "\\comma")
+				ret->wrap(ret->begin(), str_node("\\comma"));
+			ret->append_child(ret->begin(), ex2->begin());
+
+			auto it = ret->begin();
+			cleanup_dispatch(*get_kernel_from_scope(), *ret, it);
+			return ret;
+			}
+		}
+	
 	Ex_ptr Ex_mul(const Ex_ptr ex1, const Ex_ptr ex2)
 		{
 		return Ex_mul(ex1, ex2, ex2->begin());
@@ -652,6 +688,7 @@ namespace cadabra {
 		.def("from_sympy", &sympy::SympyBridge::import_ex)
 		;
 
+		m.def("join", &Ex_join);
 		m.def("tree", &print_tree);
 
 		m.def("map_sympy", &map_sympy_wrapper,
