@@ -36,7 +36,7 @@ void NotebookCanvas::refresh_all()
 	auto it=visualcells.begin();
 	while(it!=visualcells.end()) {
 		auto ct=it->first->cell_type;
-		if(ct==DataCell::CellType::output || ct==DataCell::CellType::latex_view) {
+		if(ct==DataCell::CellType::output || ct==DataCell::CellType::verbatim || ct==DataCell::CellType::latex_view) {
 			it->second.outbox->update_image();
 			}
 		++it;
